@@ -13,6 +13,7 @@
     grid
     grid_surface
     lines
+    multiprocessing
     organize
     property
     rq_import
@@ -25,9 +26,8 @@
     olio
 """
 
-try:
-    # Version dynamically extracted from git tags when package is built
-    from .version import version as __version__  # type: ignore
+import logging
 
-except ImportError:
-    __version__ = "0.0.0-version-not-available"
+__version__ = "3.0.4"  # Set at build time
+log = logging.getLogger(__name__)
+log.info(f"Imported resqpy version {__version__}")
