@@ -1,9 +1,5 @@
 """_xml.py: functions to support xml creation methods in the Model class."""
 
-# following should be kept in line with major.minor tag values in repository
-citation_format = 'bp:resqpy:1.5'
-use_version_string = False
-
 import logging
 
 log = logging.getLogger(__name__)
@@ -18,6 +14,11 @@ import resqpy.olio.uuid as bu
 import resqpy.olio.xml_et as rqet
 from resqpy.olio.xml_namespaces import curly_namespace as ns
 from resqpy.olio.xml_namespaces import namespace as ns_url
+from resqpy import __version__
+
+# following should be kept in line with major.minor tag values in repository
+citation_format = f'bp:resqpy:v{__version__}'
+use_version_string = False
 
 
 def _create_tree_if_none(model):
@@ -76,13 +77,16 @@ def _new_obj_node(flavour, name_space = 'resqml2', is_top_lvl_obj = True):
 def _referenced_node(model, ref_node, consolidate = False):
     """For a given xml reference node, returns the node for the object referred to, if present."""
 
+    # log.debug(f'ref node called for: {ref_node}')
     if ref_node is None:
         return None
-    #      content_type = rqet.find_tag_text(ref_node, 'ContentType')
+    # content_type = rqet.find_tag_text(ref_node, 'ContentType')
+    # log.debug(f'ref node title: {rqet.citation_title_for_node(rqet.find_tag(ref_node, "Title"))}')
     uuid = bu.uuid_from_string(rqet.find_tag_text(ref_node, 'UUID'))
+    # log.debug(f'ref node uuid: {uuid}')
     if uuid is None:
         return None
-    #      return model.root_for_part(model.parts_list_of_type(type_of_interest = content_type, uuid = uuid))
+    # return model.root_for_part(model.parts_list_of_type(type_of_interest = content_type, uuid = uuid))
     if consolidate and model.consolidation is not None and uuid in model.consolidation.map:
         resident_uuid = model.consolidation.map[uuid]
         if resident_uuid is None:
@@ -99,6 +103,7 @@ def _referenced_node(model, ref_node, consolidate = False):
                     title_node.text = str(title)
     else:
         node = model.root_for_part(model.part_for_uuid(uuid))
+    # log.debug(f'ref_node return node: {node}')
     return node
 
 
@@ -186,7 +191,7 @@ def _create_citation(root = None, title = '', originator = None):
 
     title_node = rqet.SubElement(citation, ns['eml'] + 'Title')
     title_node.set(ns['xsi'] + 'type', ns['eml'] + 'DescriptionString')
-    title_node.text = title
+    title_node.text = str(title)
 
     originator_node = rqet.SubElement(citation, ns['eml'] + 'Originator')
     if originator is None:
@@ -261,46 +266,11 @@ def _create_doc_props(model, add_as_part = True, root = None, originator = None)
     return dp
 
 
-def _create_crs(model,
-                add_as_part = True,
-                title = 'cell grid local CRS',
-                epsg_code = None,
-                originator = None,
-                x_offset = 0.0,
-                y_offset = 0.0,
-                z_offset = 0.0,
-                areal_rotation_radians = 0.0,
-                xy_units = 'm',
-                z_units = 'm',
-                z_inc_down = True):
-    """DEPRECATED: Creates a Coordinate Reference System node and optionally adds as child of root and/or to parts forest."""
-
-    crs = rqc.Crs(model,
-                  x_offset = x_offset,
-                  y_offset = y_offset,
-                  z_offset = z_offset,
-                  rotation = areal_rotation_radians,
-                  xy_units = xy_units,
-                  z_units = z_units,
-                  z_inc_down = z_inc_down,
-                  epsg_code = epsg_code)
-
-    crs_node = crs.create_xml(add_as_part = add_as_part, title = title, originator = originator)
-
-    if model.crs_uuid is None:
-        model.crs_uuid = crs.uuid
-
-    return crs_node
-
-
-def _create_crs_reference(model, crs_root = None, root = None, crs_uuid = None):
+def _create_crs_reference(model, root = None, crs_uuid = None):
     """Creates a node refering to an existing crs node and optionally adds as child of root."""
 
-    if crs_uuid is None:
-        warnings.warn('use of crs_root is deprecated in Model.create_crs_reference(); use crs_uuid instead')
-        crs_uuid = rqet.uuid_for_part_root(crs_root)
-    else:
-        crs_root = model.root_for_uuid(crs_uuid)
+    assert crs_uuid is not None
+    crs_root = model.root_for_uuid(crs_uuid)
     assert crs_root is not None
 
     return _create_ref_node('LocalCrs',
@@ -565,8 +535,9 @@ def _create_reciprocal_relationship(model, node_a, rel_type_a, node_b, rel_type_
     if avoid_duplicates:
         existing_rel_nodes = rqet.list_of_tag(rel_root_a, 'Relationship')
         for existing in existing_rel_nodes:
-            if (rqet.stripped_of_prefix(existing.attrib['Type']) == rel_type_a and
-                    existing.attrib['Target'] == part_name_b):
+            # if (rqet.stripped_of_prefix(existing.attrib['Type']) == rel_type_a and
+            #         existing.attrib['Target'] == part_name_b):
+            if existing.attrib['Target'] == part_name_b:
                 create_a = False
                 break
     if create_a:
@@ -580,8 +551,9 @@ def _create_reciprocal_relationship(model, node_a, rel_type_a, node_b, rel_type_
     if avoid_duplicates:
         existing_rel_nodes = rqet.list_of_tag(rel_root_b, 'Relationship')
         for existing in existing_rel_nodes:
-            if (rqet.stripped_of_prefix(existing.attrib['Type']) == rel_type_b and
-                    existing.attrib['Target'] == part_name_a):
+            # if (rqet.stripped_of_prefix(existing.attrib['Type']) == rel_type_b and
+            #         existing.attrib['Target'] == part_name_a):
+            if existing.attrib['Target'] == part_name_a:
                 create_b = False
                 break
     if create_b:
