@@ -6,16 +6,16 @@ a vector is a one dimensional numpy array with 3 elements: x, y, z.
 some functions accept a tuple or list of 3 elements as an alternative to a numpy array.
 """
 
-version = '15th November 2021'
+version = '30th June 2022'
 
 import logging
 
 log = logging.getLogger(__name__)
-log.debug('vector_utilities.py version %s', version)
 
 import math as maths
-
 import numpy as np
+import numba  # type: ignore
+from numba import njit
 
 
 def radians_from_degrees(deg):
@@ -93,6 +93,21 @@ def unit_vectors(v):
     return result
 
 
+def nan_unit_vectors(v):
+    """Returns vectors with same direction as those in v but with unit length, allowing NaNs."""
+    assert v.shape[-1] == 3
+    vf = v.reshape((-1, 3))
+    nan_mask = np.isnan(vf)
+    restore = np.seterr(all = 'ignore')
+    scaling = np.sqrt(np.sum(vf * vf, axis = -1))
+    zero_mask = np.zeros(vf.shape, dtype = bool)
+    zero_mask[np.where(scaling == 0.0), :] = True
+    result = np.where(zero_mask, 0.0, vf / np.expand_dims(scaling, -1))
+    result = np.where(nan_mask, np.nan, result)
+    np.seterr(**restore)
+    return result.reshape(v.shape)
+
+
 def unit_vector_from_azimuth(azimuth):
     """Returns horizontal unit vector in compass bearing given by azimuth (x = East, y = North)."""
     azimuth = azimuth % 360.0
@@ -148,12 +163,31 @@ def azimuths(va):  # 'azimuth' is synonymous with 'compass bearing'
 
 def inclination(v):
     """Returns the inclination in degrees of v (angle relative to +ve z axis)."""
-    assert 2 <= len(v) <= 3
+    assert len(v) == 3
     unit_v = unit_vector(v)
-    radians = maths.acos(dot_product(unit_v, np.array((0.0, 0.0, 1.0))))
+    radians = maths.acos(unit_v[2])
     return degrees_from_radians(radians)
 
 
+def inclinations(a):
+    """Returns the inclination in degrees of each vector in a (angle relative to +ve z axis)."""
+    assert a.ndim > 1 and a.shape[-1] == 3
+    unit_vs = unit_vectors(a)
+    radians = np.arccos(unit_vs[..., 2])
+    return degrees_from_radians(radians)
+
+
+def nan_inclinations(a, already_unit_vectors = False):
+    """Returns the inclination in degrees of each vector in a (angle relative to +ve z axis), allowing NaNs."""
+    assert a.ndim > 1 and a.shape[-1] == 3
+    unit_vs = a if already_unit_vectors else nan_unit_vectors(a)
+    restore = np.seterr(all = 'ignore')
+    radians = np.where(np.isnan(unit_vs[..., 2]), np.nan, np.arccos(unit_vs[..., 2]))
+    result = np.where(np.isnan(radians), np.nan, degrees_from_radians(radians))
+    np.seterr(**restore)
+    return result
+
+
 def points_direction_vector(a, axis):
     """Returns an average direction vector based on first and last non-NaN points or slices in given axis."""
 
@@ -236,7 +270,7 @@ def manhatten_distance(p1, p2):
     return abs(p2[0] - p1[0]) + abs(p2[1] - p1[1]) + abs(p2[2] - p1[2])
 
 
-def manhattan_distance(p1, p2):  # alternative spelling to above
+def manhattan_distance(p1, p2):  #  alternative spelling to above
     """Returns the Manhattan distance between two points."""
     return abs(p2[0] - p1[0]) + abs(p2[1] - p1[1]) + abs(p2[2] - p1[2])
 
@@ -254,7 +288,7 @@ def degrees_difference(a, b):
 
 
 def rotation_matrix_3d_axial(axis, angle):
-    """Retuns a rotation matrix which will rotate points about axis (0, 1, or 2) by angle in degrees."""
+    """Returns a rotation matrix which will rotate points about axis (0, 1, or 2) by angle in degrees."""
 
     axis_a = (axis + 1) % 3
     axis_b = (axis_a + 1) % 3
@@ -271,7 +305,7 @@ def rotation_matrix_3d_axial(axis, angle):
 
 
 def rotation_3d_matrix(xzy_axis_angles):
-    """Retuns a rotation matrix which will rotate points about 3 axes by angles in degrees."""
+    """Returns a rotation matrix which will rotate points about 3 axes by angles in degrees."""
 
     matrix = np.zeros((3, 3))
     for axis in range(3):
@@ -281,6 +315,12 @@ def rotation_3d_matrix(xzy_axis_angles):
     return matrix
 
 
+def reverse_rotation_3d_matrix(xzy_axis_angles):
+    """Returns a rotation matrix which will rotate back points about 3 axes by angles in degrees."""
+
+    return rotation_3d_matrix(xzy_axis_angles).T
+
+
 def rotate_vector(rotation_matrix, vector):
     """Returns the rotated vector."""
 
@@ -342,11 +382,9 @@ def tilt_3d_matrix(azimuth, dip):
 def tilt_points(pivot_xyz, azimuth, dip, points):
     """Modifies array of xyz points in situ to apply dip in direction of azimuth, about pivot point."""
 
-    #   log.debug('pivot xyz: ' + str(pivot_xyz))
     matrix = tilt_3d_matrix(azimuth, dip)
     points_shape = points.shape
     points[:] -= pivot_xyz
-    #   log.debug('points shape: ' + str(points.shape))
     points[:] = np.matmul(matrix, points.reshape((-1, 3)).transpose()).transpose().reshape(points_shape)
     points[:] += pivot_xyz
 
@@ -488,6 +526,85 @@ def points_in_triangles(p, t, da, projection = 'xy', edged = False):
         return np.all(cwtd < 0.0, axis = 2)
 
 
+@njit
+def point_in_polygon(x, y, polygon):
+    """Calculates if a point in within a polygon in 2D.
+    
+    Args:
+        x (float): the point's x-coordinate.
+        y (float): the point's y-coordinate.
+        polygon (np.ndarray): array of the polygon's vertices in 2D.
+    
+    Returns:
+        inside (bool): True if point is within the polygon, False otherwise.
+    """
+    polygon_vertices = len(polygon)
+    inside = False
+    p2x = 0.0
+    p2y = 0.0
+    xints = 0.0
+    p1x, p1y = polygon[0]
+    for i in numba.prange(polygon_vertices + 1):
+        p2x, p2y = polygon[i % polygon_vertices]
+        if y > min(p1y, p2y):
+            if y <= max(p1y, p2y):
+                if x <= max(p1x, p2x):
+                    if p1y != p2y:
+                        xints = (y - p1y) * (p2x - p1x) / (p2y - p1y) + p1x
+                    if p1x == p2x or x <= xints:
+                        inside = not inside
+        p1x, p1y = p2x, p2y
+    return inside
+
+
+@njit
+def points_in_polygon(points, polygon):
+    """Calculates which points are within a polygon in 2D.
+    
+    Args:
+        points (np.ndarray): array of the points in 2D.
+        polygon (np.ndarray): array of the polygon's vertices in 2D.
+    
+    Returns:
+        polygon_points (np.ndarray): boolean array of which points are within the polygon.
+    """
+    polygon_points = np.empty(len(points), dtype = numba.boolean)
+    for point_num in numba.prange(0, len(polygon_points)):
+        polygon_points[point_num] = point_in_polygon(points[point_num, 0], points[point_num, 1], polygon)
+    return polygon_points
+
+
+@njit(parallel = True)
+def points_in_polygons(points: np.ndarray, polygons: np.ndarray) -> np.ndarray:
+    """Calculates which points are within which polygons in 2D.
+    
+    Args:
+        points (np.ndarray): array of the points in 2D.
+        polygons (np.ndarray): array of each polygons' vertices in 2D.
+    
+    Returns:
+        polygons_points (np.ndarray): boolean array of which points are within each polygon.
+    """
+    polygons_points = np.empty((len(polygons), len(points)), dtype = numba.boolean)
+    for polygon_num in numba.prange(0, len(polygons)):
+        polygons_points[polygon_num] = points_in_polygon(points, polygons[polygon_num])
+    return polygons_points
+
+
+def triangle_normal_vector(p3):
+    """For a triangle in 3D space, defined by 3 vertex points, returns a unit vector normal to the plane of the triangle."""
+
+    # todo: handle degenerate triangles
+    return unit_vector(cross_product(p3[0] - p3[1], p3[0] - p3[2]))
+
+
+@njit
+def triangle_normal_vector_numba(points):
+    """For a triangle in 3D space, defined by 3 vertex points, returns a unit vector normal to the plane of the triangle."""
+    v = np.cross(points[0] - points[1], points[0] - points[2])
+    return v / np.linalg.norm(v)
+
+
 def in_circumcircle(a, b, c, d):
     """Returns True if point d lies within the circumcircle pf ccw points a, b, c, projected onto xy plane.
 
@@ -506,7 +623,27 @@ def in_circumcircle(a, b, c, d):
 def point_distance_to_line_2d(p, l1, l2):
     """Ignoring any z values, returns the xy distance of point p from line passing through l1 and l2."""
 
-    return (abs(p[0] * (l1[1] - l2[1]) + l1[0] * (l2[1] - p[1]) + l2[0] * (p[1] - l1[1])) / naive_2d_length(l2 - l1))
+    if np.all(l2[:2] == l1[:2]):
+        return naive_2d_length(p[:2] - l1[:2])
+    return (abs(p[0] * (l1[1] - l2[1]) + l1[0] * (l2[1] - p[1]) + l2[0] * (p[1] - l1[1])) /
+            naive_2d_length(l2[:2] - l1[:2]))
+
+
+def point_distance_to_line_segment_2d(p, l1, l2):
+    """Ignoring any z values, returns the xy distance of point p from line segment between l1 and l2."""
+
+    if is_obtuse_2d(l1, p, l2):
+        return naive_2d_length(p[:2] - l1[:2])
+    elif is_obtuse_2d(l2, p, l1):
+        return naive_2d_length(p[:2] - l2[:2])
+    else:
+        return point_distance_to_line_2d(p, l1, l2)
+
+
+def is_obtuse_2d(p, p1, p2):
+    """Returns True if the angle at point p subtended by points p1 and p2, in xy plane, is greater than 90 degrees; else False."""
+
+    return np.dot((p1[:2] - p[:2]), (p2[:2] - p[:2])) < 0.0
 
 
 def isclose(a, b, tolerance = 1.0e-6):
