@@ -1,14 +1,13 @@
 """Module defining dictionary of nexus WELLSPEC column keywords"""
 
-version = '29th April 2021'
+version = "19th April 2022"
 
 # Nexus is a registered trademark of the Halliburton Company
 
 import logging
-from typing import Any, Dict, Tuple, Type
+from typing import Any, Dict, Tuple, Type, Optional, List, Union, TextIO
 
 log = logging.getLogger(__name__)
-log.debug('wellspec_keywords.py version %s', version)
 
 import numpy as np
 import pandas as pd
@@ -126,80 +125,101 @@ wellspec_dtype['DZ']       = float
 # yapf: enable
 
 
-def increment_complaints(keyword):
+def increment_complaints(keyword):  # pragma: no cover
     """Increments the count of complaints (warnings) associated with the keyword."""
 
     global wellspec_dict
-    assert (keyword.upper() in wellspec_dict.keys())
+    assert keyword.upper() in wellspec_dict.keys()
     old_entry = wellspec_dict[keyword.upper()]
-    wellspec_dict[keyword.upper()] = (old_entry[0] + 1, old_entry[1], old_entry[2], old_entry[3], old_entry[4])
+    wellspec_dict[keyword.upper()] = (
+        old_entry[0] + 1,
+        old_entry[1],
+        old_entry[2],
+        old_entry[3],
+        old_entry[4],
+    )
 
 
-def known_keyword(keyword):
+def known_keyword(keyword):  # pragma: no cover
     """Returns True if the keyword exists in the wellspec dictionary."""
 
     return keyword.upper() in wellspec_dict.keys()
 
 
-def add_unknown_keyword(keyword):
+def add_unknown_keyword(keyword):  # pragma: no cover
     """Adds the keyword to the dictionary with attributes flagged as unknown."""
 
     global wellspec_dict
-    assert (not known_keyword(keyword))
-    wellspec_dict[keyword.upper()] = (1, wk_unknown, wk_banned, None, False)  # assumes warning or error already given
+    assert not known_keyword(keyword)
+    wellspec_dict[keyword.upper()] = (
+        1,
+        wk_unknown,
+        wk_banned,
+        None,
+        False,
+    )  # assumes warning or error already given
 
 
-def default_value(keyword):
+def default_value(keyword):  # pragma: no cover
     """Returns the default value for the keyword."""
 
-    assert (known_keyword(keyword))
+    assert known_keyword(keyword)
     return wellspec_dict[keyword][3]
 
 
-def complaints(keyword):
+def complaints(keyword):  # pragma: no cover
     """Returns the number of complaints (warnings) logged for the keyword."""
 
-    assert (known_keyword(keyword))
+    assert known_keyword(keyword)
     return wellspec_dict[keyword][0]
 
 
 def check_value(keyword, value):
     """Returns True if the value is acceptable for the keyword."""
-
     try:
         key = keyword.upper()
         if not known_keyword(key):
             return False
-        if key in ['IW', 'JW', 'L', 'LAYER', 'IRELPM', 'CELL', 'SECT', 'FLOWSECT', 'ZONE', 'IPTN']:
+        if key in [
+                "IW",
+                "JW",
+                "L",
+                "LAYER",
+                "IRELPM",
+                "CELL",
+                "SECT",
+                "FLOWSECT",
+                "ZONE",
+                "IPTN",
+        ]:
             return int(value) > 0
-        elif key == 'GRID':
+        elif key == "GRID":
             return len(str(value)) > 0
-        elif key == 'STAT':
-            return (str(value)).upper() in ['ON', 'OFF']
-        elif key == 'ANGLA':
+        elif key == "STAT":
+            return (str(value)).upper() in ["ON", "OFF"]
+        elif key == "ANGLA":
             return -360.0 <= float(value) and float(value) <= 360.0
-        elif key == 'ANGLV':
+        elif key == "ANGLV":
             return 0.0 <= float(value) and float(value) <= 180.0
-        elif key in ['RADW', 'RADB', 'RADWP', 'RADBP']:
+        elif key in ["RADW", "RADB", "RADWP", "RADBP"]:
             return float(value) > 0.0
-        elif key in ['WI', 'LENGTH', 'KH', 'KHMULT', 'K', 'DZ']:
+        elif key in ["WI", "LENGTH", "KH", "KHMULT", "K", "DZ"]:
             return float(value) >= 0.0
-        elif key == 'PPERF':
+        elif key == "PPERF":
             return 0.0 <= float(value) and float(value) <= 1.0
-        elif key == 'ANGLE':
+        elif key == "ANGLE":
             return 0.0 <= float(value) and float(value) <= 360.0
-        elif key in ['SKIN', 'DEPTH', 'X', 'Y', 'TEMP']:
+        elif key in ["SKIN", "DEPTH", "X", "Y", "TEMP"]:
             float(value)
             return True
-        else:
+        else:  # pragma: no cover
             return True
     except Exception:
         return False
 
 
-def required_out_list():
+def required_out_list():  # pragma: no cover
     """Returns a list of keywords that are required."""
-
     list = []
     for key in wellspec_dict.keys():
         if wellspec_dict[key][2] == wk_required:
@@ -207,109 +227,165 @@ def required_out_list():
     return list
 
 
-def length_unit_conversion_applicable(keyword):
+def length_unit_conversion_applicable(keyword):  # pragma: no cover
     """Returns True if the keyword has a quantity class of length."""
 
-    assert (known_keyword(keyword))
+    assert known_keyword(keyword)
     return wellspec_dict[keyword][4]
 
 
-def load_wellspecs(wellspec_file, well = None, column_list = []):
+def load_wellspecs(
+    wellspec_file: str,
+    well: Optional[str] = None,
+    column_list: Union[List[str], None] = [],
+) -> Dict[str, Union[pd.DataFrame, None]]:
     """Reads the Nexus wellspec file returning a dictionary of well name to pandas dataframe.
 
-    args:
-       wellspec_file (string): file path of ascii input file containing wellspec keywords
-       well (optional, string): if present, only the data for the named well are loaded;
-          if None, data for all wells are loaded
-       column_list (list of strings, optional): if present, each dataframe returned contains
-          these columns, in this order; if None, the resulting dictionary contains only
-          well names as keys (each mapping to None rather than a dataframe); if an empty list,
-          each dataframe contains the columns listed in the corresponding wellspec header, in
-          the order found in the file
+    Args:
+       wellspec_file (str): file path of ascii input file containing wellspec keywords.
+       well (str, optional): if present, only the data for the named well are loaded. If None, data
+          for all wells are loaded.
+       column_list (List[str]/None): if present, each dataframe returned contains these
+          columns, in this order. If None, the resulting dictionary contains only well names as keys
+          (each mapping to None rather than a dataframe). If an empty list, each dataframe contains
+          the columns listed in the corresponding wellspec header, in the order found in the file.
 
-    returns:
-       dictionary (string: pandas dataframe) mapping each well name found in the wellspec file
-          to a dataframe containing the wellspec data
+    Returns:
+       well_dict (Dict[str, Union[pd.DataFrame, None]]): mapping each well name found in the
+          wellspec file to a dataframe containing the wellspec data.
     """
-
-    assert wellspec_file, 'no wellspec file specified'
+    assert wellspec_file, "no wellspec file specified"
 
     if column_list is not None:
         for column in column_list:
-            assert column.upper() in wellspec_dict, 'unrecognized wellspec column name ' + str(column)
+            assert (column.upper() in wellspec_dict), "unrecognized wellspec column name " + str(column)
     selecting = bool(column_list)
 
-    well_dict = {}  # maps from well name to pandas data frame with column_list as columns
+    well_dict = {}
+    well_pointers = get_well_pointers(wellspec_file)
 
-    with open(wellspec_file, 'r') as fp:
-        while True:
-            found = kf.find_keyword(fp, 'WELLSPEC')
-            if not found:
-                break
-            line = fp.readline()
-            words = line.split()
-            assert len(words) >= 2, 'missing well name after WELLSPEC keyword'
-            well_name = words[1]
-            if well and well_name.upper() != well.upper():
-                continue
-            if column_list is None:
-                well_dict[well_name] = None
-                continue
-            kf.skip_blank_lines_and_comments(fp)
-            line = kf.strip_trailing_comment(fp.readline()).upper()
-            columns_present = line.split()
-            if selecting:
-                column_map = np.full((len(column_list),), -1, dtype = int)
-                for i in range(len(column_list)):
-                    column = column_list[i].upper()
-                    if column in columns_present:
-                        column_map[i] = columns_present.index(column)
-                df = pd.DataFrame(columns = column_list)
-            else:
-                df = pd.DataFrame(columns = columns_present)
-            while True:
-                kf.skip_comments(fp)
-                if kf.blank_line(fp):
-                    break  # unclear from Nexus doc what marks end of table
-                if kf.specific_keyword_next(fp, 'WELLSPEC') or kf.specific_keyword_next(fp, 'WELLMOD'):
-                    break
-                line = kf.strip_trailing_comment(fp.readline())
-                words = line.split()
-                assert len(words) >= len(
-                    columns_present), f'insufficient data in line of wellspec table {well} [{line}]'
-                row_dict = {}
-                if selecting:
-                    for col_index in range(len(column_list)):
-                        column = column_list[col_index]
-                        if column_map[col_index] < 0:
-                            if column_list[col_index].upper() == 'GRID':
-                                row_dict[column] = 'ROOT'
-                            else:
-                                row_dict[column] = np.NaN
-                        else:
-                            v = words[column_map[col_index]]
-                            if v == 'NA':
-                                row_dict[column] = np.NaN
-                            elif v == '#':
-                                row_dict[column] = v
-                            else:
-                                row_dict[column] = wellspec_dtype[column.upper()](v)
-                else:
-                    for column, v in zip(columns_present, words[:len(columns_present)]):
-                        if v == 'NA':
-                            row_dict[column] = np.NaN
-                        elif v == '#':
-                            row_dict[column] = v
-                        else:
-                            row_dict[column] = wellspec_dtype[column](v)
-                df = df.append(row_dict, ignore_index = True)
+    if column_list is None:
+        well_dict = dict.fromkeys(well_pointers, None)
+        return well_dict
 
-            if well:
-                well_dict[well] = df
-                break  # NB. if more than one table for a well, this function returns first, Nexus uses last
-            well_dict[well_name] = df
-
-
-#   log.debug(f'load-wellspecs returning:\n{well_dict}')
+    with open(wellspec_file, "r") as file:
+        if well:
+            well_data = get_well_data(file, well, well_pointers[well], column_list, selecting)
+            if well_data is not None:
+                well_dict[well] = well_data
+        else:
+            for well_name, pointer in well_pointers.items():
+                well_data = get_well_data(file, well_name, pointer, column_list, selecting)
+                if well_data is not None:
+                    well_dict[well_name] = well_data
 
     return well_dict
+
+
+def get_well_pointers(wellspec_file: str) -> Dict[str, int]:
+    """Gets the file locations of each well in the wellspec file for optimised processing of the data.
+
+    Args:
+        wellspec_file (str): file path of ascii input file containing wellspec keywords.
+
+    Returns:
+        well_pointers (Dict[str, int]): mapping each well name found in the wellspec file to their
+            file location.
+    """
+    well_pointers = {}
+    with open(wellspec_file, "r") as file:
+        while True:
+            found = kf.find_keyword(file, "WELLSPEC")
+            if not found:
+                break
+            line = file.readline()
+            words = line.split()
+            assert len(words) >= 2, "missing well name after WELLSPEC keyword"
+            well_name = words[1]
+            well_pointers[well_name] = file.tell()
+
+    return well_pointers
+
+
+def get_well_data(
+    file: TextIO,
+    well_name: str,
+    pointer: int,
+    column_list: List[str],
+    selecting: bool,
+) -> Union[pd.DataFrame, None]:
+    """Creates a dataframe of the well data for a given well name in the wellspec file.
+
+    The pointer argument is used to go to the file location where the well data is located.
+
+    Args:
+        file (TextIO): the opened wellspec file object.
+        well_name (str): name of the well.
+        pointer (int): the file object's start position of the well data represented as number of
+            bytes from the beginning of the file.
+        column_list (List[str]): if present, each dataframe returned contains these
+            columns, in this order. If None, the resulting dictionary contains only well names as keys
+            (each mapping to None rather than a dataframe). If an empty list, each dataframe contains
+            the columns listed in the corresponding wellspec header, in the order found in the file.
+        selecting (bool): True if the column_list contains at least one column name, False otherwise.
+
+    Returns:
+        Pandas dataframe of the well data or None if there is a nan in the last row.
+    """
+    file.seek(pointer)
+    kf.skip_blank_lines_and_comments(file)
+    line = kf.strip_trailing_comment(file.readline()).upper()
+    columns_present = line.split()
+    if selecting:
+        column_map = np.full((len(column_list),), -1, dtype = int)
+        for i in range(len(column_list)):
+            column = column_list[i].upper()
+            if column in columns_present:
+                column_map[i] = columns_present.index(column)
+        df_col = column_list
+    else:
+        df_col = columns_present
+    data: Dict[str, List] = {col: [] for col in df_col}
+    all_null = True
+    while True:
+        kf.skip_comments(file)
+        if kf.blank_line(file):
+            break  # unclear from Nexus doc what marks end of table
+        if kf.specific_keyword_next(file, "WELLSPEC") or kf.specific_keyword_next(file, "WELLMOD"):
+            break
+        line = kf.strip_trailing_comment(file.readline())
+        words = line.split()
+        assert len(words) >= len(columns_present), f"insufficient data in line of wellspec table {well_name} [{line}]"
+        if selecting:
+            for col_index, col in enumerate(column_list):
+                if column_map[col_index] < 0:
+                    if column_list[col_index].upper() == "GRID":
+                        data[col].append("ROOT")
+                    else:
+                        data[col].append(np.NaN)
+                else:
+                    value = words[column_map[col_index]]
+                    if value == "NA":
+                        data[col].append(np.NaN)
+                    elif value == "#":
+                        data[col].append(value)
+                    elif value:
+                        data[col].append(wellspec_dtype[col.upper()](value))
+                if not pd.isnull(data[col][-1]):
+                    all_null = False
+        else:
+            for col, value in zip(columns_present, words[:len(columns_present)]):
+                if value == "NA":
+                    data[col].append(np.NaN)
+                elif value == "#":
+                    data[col].append(value)
+                elif value:
+                    data[col].append(wellspec_dtype[col](value))
+                if not pd.isnull(data[col][-1]):
+                    all_null = False
+
+    if all_null:
+        log.warning(f"skipping null wellspec data for well {well_name}")
+        return None
+
+    return pd.DataFrame(data)
